package org.openxma.dsl.generator;

import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Map;
import java.util.Properties;

import org.eclipse.emf.mwe.core.resources.ResourceLoaderFactory;

/**
 * Default base implementation of interface {@link GeneratorConfiguration} which
 * uses a simple property file override mechanism for determining the final set
 * of property values. <br/>
 * Properties are loaded and overridden in the following order: <br/>
 * <ol>
 * <li>default values from
 * org/openxma/dsl/generator/default-generator.properties located inside the
 * generator archive</li>
 * <li>/generator.properties from current classpath</li>
 * <li>file location as specified by the system property 'generator.properties'
 * (-Dgenerator.properties=file:/..)</li>
 * <li>{@link Properties} instance provided with the constructor of this class</li>
 * </ol>
 */
public abstract class AbstractGeneratorConfiguration implements
		GeneratorConfiguration {
	private static final long serialVersionUID = 1L;
	private Properties properties;
	private NamingStrategy namingStrategy;
	private ModelModifier modelModifier;
	private LayoutStrategy layoutStrategy;
	private String fileExtension;

	public AbstractGeneratorConfiguration(Properties properties) {
		initializeProperties(properties);
	}

	public String getFileExtension() {
		return fileExtension;
	}

	public void setFileExtension(String fileExtension) {
		this.fileExtension = fileExtension;
	}

	public LayoutStrategy getLayoutStrategy() {
		if (layoutStrategy == null) {
			layoutStrategy = newInstanceOf(getProperty(GeneratorConstants.PRESENTATION_MODEL_LAYOUTSTRATEGY_CLASS));
		}
		return layoutStrategy;
	}

	public ModelModifier getModelModifier() {
		if (modelModifier == null) {
			modelModifier = newInstanceOf(getProperty(GeneratorConstants.DOMAIN_MODEL_MODIFIER_CLASS));
		}
		return modelModifier;
	}

	public NamingStrategy getNamingStrategy() {
		if (namingStrategy == null) {
			namingStrategy = newInstanceOf(getProperty(GeneratorConstants.DOMAIN_MODEL_NAMINGSTRATEGY_CLASS));
		}
		return namingStrategy;
	}

	public String getProperty(String key) {
		return this.properties.getProperty(key);
	}

	public String getProperty(String key, String defaultValue) {
		return this.properties.getProperty(key, defaultValue);
	}

	private void initializeProperties(Properties properties) {
		Properties defaultProperties = new Properties();
		loadProperties(defaultProperties, GeneratorConstants.DEFAULT_PROPERTIES);
		Properties classPathProperties = new Properties(defaultProperties);
		loadProperties(classPathProperties, GeneratorConstants.CLASSPATH_PROPERTIES);
		Properties systemProperties = new Properties(classPathProperties);
		String propertyFile = System.getProperty(GeneratorConstants.CLASSPATH_PROPERTIES);
		if (propertyFile != null) {
			loadProperties(systemProperties, propertyFile);
		}
		if (properties != null) {
			mergePropertiesIntoMap(properties, systemProperties);
		}
		this.properties = systemProperties;
	}

	private void loadProperties(Properties properties, String resource) {
		InputStream resourceInputStream = ResourceLoaderFactory
				.createResourceLoader().getResourceAsStream(resource);
		if (resourceInputStream != null) {
			try {
				properties.load(resourceInputStream);
			} catch (IOException exception) {
			}
		}
	}

	private void mergePropertiesIntoMap(Properties props,
			Map<Object, Object> map) {
		if (map == null) {
			throw new IllegalArgumentException(
					"parameter 'map' must not be null");
		}
		if (props != null) {
			for (Enumeration<?> en = props.propertyNames(); en
					.hasMoreElements();) {
				String key = (String) en.nextElement();
				map.put(key, props.getProperty(key));
			}
		}
	}

	static <T> T newInstanceOf(String className) {
		T newInstance = null;
		if (className != null) {
			try {
				Class<T> clazz = loadClass(className);
				newInstance = clazz.newInstance();
			} catch (Exception exception) {
				throw new IllegalStateException(exception);
			}
		}
		return newInstance;
	}

	@SuppressWarnings("unchecked")
	static <T> Class<T> loadClass(String className)
			throws ClassNotFoundException {
		return (Class<T>) ResourceLoaderFactory.createResourceLoader()
				.loadClass(className);
	}

}
